unit Chips;
//***********************************************************************
//**  Chips Component                                                  **
//***********************************************************************
//**  This component simulates poker chips for a variety of gambling   **
//**  card games.  Readily usable in any program, it is well suited    **
//**  for use with TCardV1, a component used for drawing cards.        **
//**  The code contained in this document is entirely free to use in   **
//**  both commercial and non-commercial applications.  If this        **
//**  component is utilized in any way, shape, or form, I would        **
//**  appreciate a notification via email indicating this, as well as  **
//**  any comments or suggestions you may have.  Bug reports are       **
//**  encouraged.                                                      **
//**  I can be reached at:                                             **
//**    elvis@sway.com                                                 **
//**                                Sincerely,                         **
//**                                  T. J. Sobotka                    **
//**                                  March 1997                       **
//***********************************************************************
//**  Properties:                                                      **
//**    ChipType:  Determines the color of the chip.                   **
//**    ChipValue:  Used to denote value of chip.  Note that there     **
//**           are no methods to check the width of the numerical      **
//**           text, so use with care.  Also note that the property    **
//**           is integer based for calculation purposes.              **
//***********************************************************************
interface

{$RESOURCE 'CHIPSET'}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const
  DEFAULT_CHIP_WIDTH              = 32;
  DEFAULT_CHIP_HEIGHT             = 32;

type
  TChipType = (RedChip, BlueChip, WhiteChip);
  TChip = class(TGraphicControl)
  private
    { Private declarations }
  protected
    { Protected declarations }
    FChipValue: LongInt;
    FChipType: TChipType;
    TextColor: TColor;
    ShadowColor: TColor;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    procedure SetChipType(ChipType: TChipType);
    procedure SetChipValue(ChipValue: LongInt);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    { Published declarations }
    property ChipValue: LongInt read FChipValue write SetChipValue;
    property ChipType: TChipType read FChipType write SetChipType;
  end;

procedure Register;

var
  ChipInstanceCount: LongInt = 0;
  ChipSet: TBitmap;
  ChipMask: TBitmap;

implementation

constructor TChip.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FChipValue := 10;
  if (ChipInstanceCount < 1) then
    begin
    ChipSet := TBitmap.Create;
    ChipMask := TBitmap.Create;
    ChipMask.Monochrome := True;
    ChipSet.LoadFromResourceName(HInstance, 'CHIPSET');
    ChipMask.LoadFromResourceName(HInstance, 'CHIPMASK');
    end;
  Canvas.Brush.Style := bsClear;
  SetChipType(RedChip);
  Inc(ChipInstanceCount);
end;

destructor TChip.Destroy;
begin
  Dec(ChipInstanceCount);
  if (ChipInstanceCount < 1) then
    begin
    ChipSet.Free;
    ChipMask.Free;
    end;
  inherited Destroy;
end;

procedure TChip.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, DEFAULT_CHIP_WIDTH, DEFAULT_CHIP_HEIGHT);
end;

procedure TChip.Paint;
var
  CaptionWidth: LongInt;
  CaptionHeight: LongInt;
  ChipValueText: String;
begin
  BitBlt(Canvas.Handle, 0, 0, DEFAULT_CHIP_WIDTH, DEFAULT_CHIP_HEIGHT,
    ChipMask.Canvas.Handle, 0, 0, SRCAND);
  BitBlt(Canvas.Handle, 0, 0, DEFAULT_CHIP_WIDTH, DEFAULT_CHIP_HEIGHT,
        ChipSet.Canvas.Handle, (LongInt(FChipType) * DEFAULT_CHIP_WIDTH),
        0, SRCPAINT);

  ChipValueText := IntToStr(FChipValue);
  CaptionWidth := Canvas.TextWidth(ChipValueText);
  CaptionHeight := Canvas.TextHeight(ChipValueText);
  Canvas.Font.Color := ShadowColor;
  Canvas.TextOut((DEFAULT_CHIP_WIDTH div 2) - (CaptionWidth div 2),
    (DEFAULT_CHIP_HEIGHT div 2) - (CaptionHeight div 2), ChipValueText);
  Canvas.Font.Color := TextColor;
  Canvas.TextOut((DEFAULT_CHIP_WIDTH div 2) - (CaptionWidth div 2) - 1,
    (DEFAULT_CHIP_HEIGHT div 2) - (CaptionHeight div 2) - 1, ChipValueText);
end;

procedure TChip.SetChipType(ChipType: TChipType);
begin
  FChipType := ChipType;
  case ChipType of
    RedChip:
      begin
      TextColor := clMaroon;
      ShadowColor := clWhite;
      end;
    BlueChip:
      begin
      TextColor := clNavy;
      ShadowColor := clWhite;
      end;
    WhiteChip:
      begin
      TextColor := clGray;
      ShadowColor := clSilver;
      end;
  end;
  Refresh;
end;

procedure TChip.SetChipValue(ChipValue: LongInt);
begin
  FChipValue := ChipValue;
  Refresh;
end;

procedure Register;
begin
  RegisterComponents('Cards', [TChip]);
end;

end.
 